;INT.ASM
*******************************************************************************
*REVISION HISTORY:
*
*DATE			REV. NO.	DESCRIPTION
*
*August 15, 2005	1.00		Initial release
*
*
*Author: Exequiel Rarama for the Adapt9S12NE64
*******************************************************************************
;Assembled using CW
;
; ---------------------------
; Demo - ISR Routine
; ---------------------------


	include "NE64Regs.INC"


;Public Function
	XDEF RealTimeInit
	XDEF RealTimeInt

	XDEF OutStr0
	XDEF SerOutput0
	XDEF SerInputInt0

	XDEF audio_int

;Public Variables
	XDEF ad0
	XDEF ad1
	XDEF ad2
	XDEF ad3
	XDEF ad4
	XDEF ad5
	XDEF ad6
	XDEF ad7
	XDEF adflag

	XDEF TIMER
	XDEF TIMER1
	XDEF COUNTER

	XREF check_audio
	
;External Variables
	XREF d_flag

	XREF COMMAND
	XREF COMMAND_PENDING
	XREF CONTINUOUS
	
	XREF audio_status
	XREF audio_period

	XREF audio_delay
	XREF audio_counter

	XREF state_timer

	XREF LANTimer

DataSec:	SECTION

;ATD ram variables
atd_var:

ad0:		ds	2
ad1:		ds	2
ad2:		ds	2
ad3:		ds	2
ad4:		ds	2
ad5:		ds	2
ad6:		ds	2
ad7:		ds	2
adflag:		ds	1

TIMER:		ds	2       ;software timers using RTI
TIMER1:		ds	2
COUNTER:	ds	1

;------------------------------------------------------------------------------
;ATD Variables
admask2:         equ     %11000000       ;AFFC,ADPU=1 - Enable Analog to Digital
admask3:         equ     %00000000       ;FRZ1,FRZ0=0
admask4:         equ     %10000001       ;SMP1,SMP0 = 0; S10BM,PRS0=1 - Select Sample time adn Bit mode
admask5:         equ     %01110000       ;S8CM = 1, SCAN = 1, MULT = 1
SCFflag:         equ     %10000000       ;SCF - Sequence Complete flag

;RTI Variables
clrmask:         equ     %11000000       ;mask for clearing timer flags
RTIRate:		equ     %01110000       ;
RTIF:		equ	%10000000
RTIE:		equ	%10000000

;SCI Variables
scimask:         equ     %00101100       ;RIE - SCI Interrupt enable
                                        ;RE - Receiver Enable
RDRFflag:        equ     %00100000       ;RDRF - Receive Data Register Full flag
TDREflag:        equ     %10000000       ;TDRE - Transmit Data Register Empty flag

;Baud rate definitions
OscFreq:     	equ	25000		;Enter Osc speed
initSYNR:    	equ	$00		; mult by synr + 1 = 1 (25MHz)
initREFDV:   	equ	$00		; divide by Refdiv + 1

BusFreq:     	equ	((OscFreq/(initREFDV+1))*(initSYNR+1))
baud115200: 	equ	(BusFreq/16)*10/1152  			;sets baud rate to 115,200
baud9600:	equ	(BusFreq/16)*10/96  			;sets baud rate to 009,600
Baud:		equ	$8000+baud9600
initSCI0CR2:	equ	$0C					;SCI0 Control Register 2

* Operational Constants

TRUE:		equ	$FF
FALSE:		equ	$00
CR:		equ	$D
LF:		equ	$A
SPACE:		equ	$20

LED:		equ	$01
IREN:		equ	$80
;------------------------------------------------------------------------------

***************************** Program *******************************

NON_BANKED:SECTION
;
RealTimeInit:				;Initialize Real Time Interrupt
	movb	#RTIRate,RTICTL		;and initialize RTI rate
	bset	CRGFLG,RTIF		;clear flag
	bset	CRGINT,RTIE		;Enable RTI

;Initialize Analog To Digital
	movb	#$80,ATDCTL2	        ;enable ATD
	movb	#$40,ATDCTL3	        ;8 Channels ATD result
	movb	#$60,ATDCTL4		;Select Sample rate
	movb	#$B0,ATDCTL5		;Select 8 channel mode, Continuous scan
	clr	adflag

;Initialize first Serial Communication Interface
;	movw	#baud9600,SCI0BDH	;Set baud rate to 6900

	movb	#baud115200,SCI0BDL	;BDH=0 so baud = 115.2 K
	movb	#scimask,SCI0CR2	;Rx and Tx on
	ldab	SCI0SR1  		;read register to clear flag RDRF
	ldab	SCI0DRL			;read receive buffer

;Disable IRQ
	movb	#$00,INTCR		;Clear IRQEN bit
	ldx	#WarningMSG1		;Send warning message
	jsr	OutStr0
	rts

WarningMSG1	dc.b	$D,$A,'IRQ is disabled',$D,$A,0

;------------------------------------------------------------------------------
* Real-time Interrupt Routine

RealTimeInt:
	bset	CRGFLG,RTIF		;clear real-time interrupt flag
	cli				; so that other interrupts can be service

RTI1:
	ldx	TIMER
	beq	RTI2
	dex
	stx	TIMER

RTI2:
	ldx	TIMER1
	beq	RTI3
	dex
	stx	TIMER1

RTI3:
	ldx	audio_delay
	beq	RTI4
	dex	
	stx	audio_delay
	
RTI4:
	ldx	state_timer
	beq	RTI5
	dex	
	stx	state_timer

RTI5:
	ldx	LANTimer
	beq	RTI6
	dex	
	stx	LANTimer

RTI6:
	brclr	ATDSTAT0,SCFflag,*	;Loop here until SCF of ATD is set save ATD
	bset	ATDSTAT0,SCFflag	;clear Flag for next conversion

	ldd	ATDDR0H
	std	ad0

	ldd	ATDDR1H
	std	ad1

	ldd	ATDDR2H
	std	ad2

	ldd	ATDDR3H
	std	ad3

	ldd	ATDDR4H
	std	ad4

	ldd	ATDDR5H
	std	ad5

	ldd	ATDDR6H
	std	ad6

	ldd	ATDDR7H
	std	ad7

rtiex:
	jsr	check_audio
	rti


;==============================================================================
*  SCI Input Interrupt Handler

*  Gets bytes from SCI.  Sets COMMAND_PENDING flag.


OutStr0:				; send a null terminated string to the display.
	ldaa	1,x+            ; get a character, advance pointer, null?
	beq	OutStrDone      ; yes. return.
	bsr	SerOutput0      ; no. send it out the SCI.
	bra	OutStr0          ; go get the next character.
;
OutStrDone:
	rts

;------------------------------------------------------------------------------
SerOutput0:
	brclr	SCI0SR1,TDREflag,SerOutput0     ;check if buffer is empty
	staa	SCI0DRL
	rts

SerInputInt0:
	ldaa	SCI0SR1                  ;read register to clear flag RDRF
	movb	SCI0DRL,COMMAND          ;read receive buffer
	movb	#TRUE,COMMAND_PENDING

SIIX:
	rti



;==============================================================================
* Output Compare Initialization routine
;
OC4mask1:	equ	%00010000	;IOS4 = 1, Bit 4 as output compare
OC4mask2:	equ	%00010000	;C4I = 1, Enable Interrupt
OC4mask3:	equ	%00000001	;OM4 = 0, OL4 = 1, toggel OC output line
OC4flag:	equ	%00010000	;C4F = 1 to clear Interrupt flag
AudioControl:	equ	%00010000	;Port T bit 4

;------------------------
; Audio Interrupt Routine
;------------------------
audio_int:
	ldd	audio_period		;audio time period for next
	addd	T0TC4			;  output compare interrupt
	std	T0TC4			

	ldx	audio_period
	beq	auiex

	ldaa	audio_status
	eora	#1
	staa	audio_status
	beq	aui10

	bset	PORTT,AudioControl
	bra	auiex

aui10:
	bclr	PORTT,AudioControl

auiex:
	bset	T0TFLG1,OC4flag		;clear flag
	rti


;***********************************************************************************************

	END

